"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createAndRunApp = exports.createApp = void 0;
const express_1 = __importDefault(require("express"));
const path_1 = require("path");
const cors_1 = __importDefault(require("cors"));
const http_terminator_1 = require("http-terminator");
const express_rate_limit_1 = __importDefault(require("express-rate-limit"));
const torrent_client_1 = require("./services/torrent-client");
const config_1 = require("./config");
const logging_1 = require("./helpers/logging");
const api_1 = require("./api");
require("express-async-errors");
function createApp(options) {
    var _a, _b;
    const config = (0, config_1.readConfig)(options.configFile, options.config);
    const logger = (_a = options.logger) !== null && _a !== void 0 ? _a : (0, logging_1.createLogger)(config);
    const client = (_b = options.client) !== null && _b !== void 0 ? _b : new torrent_client_1.TorrentClient({
        logger,
        ...config.torrents,
    });
    const app = (0, express_1.default)();
    app.use((0, cors_1.default)());
    app.use(express_1.default.json());
    if (config.trustProxy) {
        app.set('trust proxy', true);
    }
    app.use((0, express_rate_limit_1.default)({
        windowMs: 60 * 1000,
        max: config.security.rpm,
    }));
    app.use((0, api_1.getApiRouter)({
        config,
        logger,
    }, client));
    if (config.security.frontendEnabled) {
        app.use(express_1.default.static(config_1.frontendBuildPath));
        app.use((_req, res) => {
            res.sendFile((0, path_1.join)(config_1.frontendBuildPath, 'index.html'));
        });
    }
    return { app, client, logger, config };
}
exports.createApp = createApp;
function createAndRunApp(options) {
    const { app, config, logger, client } = createApp(options);
    const server = app.listen(config.port, config.host, () => {
        logger.info(`Running ${config.environment} server on ${config.host}:${config.port}`);
        if (config.environment === 'development') {
            const accessHost = config.host === '0.0.0.0' ? '127.0.0.1' : config.host;
            logger.info(`* Website on http://${accessHost}:${config.port}`);
            logger.info(`* Docs on http://${accessHost}:${config.port}/api-docs`);
        }
    });
    const httpTerminator = (0, http_terminator_1.createHttpTerminator)({
        server,
    });
    const shutdown = async (signal) => {
        logger.info(`${signal} signal received, closing server`);
        try {
            await httpTerminator.terminate();
        }
        catch (err) {
            logger.warn(`Error while terminating HTTP server: ${err}`);
        }
        try {
            await client.destroy();
        }
        catch (err) {
            logger.warn(`Error while terminating torrents client: ${err}`);
        }
        process.exit();
    };
    process.on('SIGTERM', shutdown);
    process.on('SIGINT', shutdown);
    return { app, config, logger, client };
}
exports.createAndRunApp = createAndRunApp;
