"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TorrentClient = void 0;
const util_1 = require("util");
const parse_torrent_1 = __importDefault(require("parse-torrent"));
const common_stuff_1 = require("common-stuff");
const mime_types_1 = require("mime-types");
const _1 = require(".");
class TorrentClient {
    constructor(config, adapter) {
        this.torrents = {};
        this.cleanLocked = false;
        this.config = config;
        this.adapter =
            adapter !== null && adapter !== void 0 ? adapter : new _1.WebtorrentAdapter({
                downloadLimit: config.downloadLimit,
                uploadLimit: config.uploadLimit,
            });
        this.isLoading = config.useDefaultTrackers
            ? (0, _1.downloadTrackers)()
                .then((v) => {
                var _a;
                config.logger.debug(`Loaded ${v.length} default trackers`);
                this.config = {
                    ...this.config,
                    announce: (0, common_stuff_1.deduplicate)([
                        ...((_a = config.announce) !== null && _a !== void 0 ? _a : []),
                        ...v,
                    ]),
                };
            })
                .catch(() => {
                config.logger.warn('Failed to load tracker list');
            })
            : Promise.resolve();
    }
    async loadAndWait() {
        await this.isLoading;
    }
    getTorrents() {
        return Object.values(this.torrents);
    }
    getTorrent(infoHash) {
        return this.torrents[infoHash];
    }
    async removeTorrent(infoHash) {
        await this.isLoading;
        const torrent = this.torrents[infoHash];
        if (torrent) {
            await torrent.remove();
            delete this.torrents[infoHash];
        }
    }
    async addTorrent(link) {
        await this.isLoading;
        let parsedLink;
        try {
            parsedLink = await (0, util_1.promisify)(parse_torrent_1.default.remote)(link);
        }
        catch (err) {
            throw new _1.TorrentClientError(`Cannot parse torrent: ${err instanceof Error ? err.message : err}, link: ${link}`);
        }
        if (!parsedLink) {
            throw new _1.TorrentClientError(`Cannot parse torrent: ${link}`);
        }
        const magnet = parse_torrent_1.default.toMagnetURI({
            ...parsedLink,
            announce: (0, common_stuff_1.deduplicate)([
                ...(parsedLink.announce || []),
                ...(this.config.announce || []),
            ]),
            urlList: (0, common_stuff_1.deduplicate)([
                ...(parsedLink.urlList || []),
                ...(this.config.urlList || []),
            ]),
            // @ts-ignore
            peerAddresses: (0, common_stuff_1.deduplicate)([
                // @ts-ignore
                ...(parsedLink.peerAddresses || []),
                ...(this.config.peerAddresses || []),
            ]),
        });
        const infoHash = parsedLink.infoHash;
        const foundTorrent = this.torrents[infoHash];
        if (foundTorrent) {
            this.torrents[infoHash] = {
                ...foundTorrent,
                updated: new Date(),
            };
            return foundTorrent;
        }
        const torrent = await this.adapter
            .add(magnet, this.config.path)
            .then((v) => ({
            ...v,
            link,
            infoHash,
            created: new Date(),
            updated: new Date(),
            files: v.files.map((f) => ({
                ...f,
                type: (0, mime_types_1.lookup)(f.name) || '',
            })),
        }));
        this.torrents[torrent.infoHash] = torrent;
        setTimeout(() => {
            this.checkForExpiredTorrents().catch((err) => {
                this.config.logger.error(err);
            });
        }, 1000);
        return torrent;
    }
    async destroy() {
        await this.isLoading;
        this.config.logger.info('Closing torrent client');
        await this.adapter.destroy();
    }
    async checkForExpiredTorrents() {
        await this.isLoading;
        if (this.cleanLocked) {
            return;
        }
        this.cleanLocked = true;
        try {
            const torrentToRemove = Object.values(this.torrents).filter((torrent) => Date.now() - torrent.updated.getTime() >
                this.config.ttl * 1000);
            for (const torrent of torrentToRemove) {
                this.config.logger.info(`Removing expired ${torrent.name} torrent`);
                await this.removeTorrent(torrent.infoHash);
            }
        }
        finally {
            this.cleanLocked = false;
        }
    }
}
exports.TorrentClient = TorrentClient;
